/* 
 * File:   optimalizer.c
 * Author: Michal Hucik <hucik@ordoz.com>
 *
 * Created on 27. května 2019, 15:03
 * 
 * 
 * ----------------------------- License -------------------------------------
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * ---------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <glib.h>


#include "gdgelement.h"
#include "gdgpart.h"
#include "gdgnet.h"


static void optimalizer_remove_buffers ( st_GDGELEMENTSALL *elmall, st_GDGPARTSALL *partall, st_GDGNETSALL *netall ) {

    int remove_part_count = 0;
    st_GDGPART **remove_part = g_new0 ( st_GDGPART*, 1 );

    int i;
    for ( i = 0; i < partall->count_part; i++ ) {
        st_GDGPART *part = partall->part[i];
        st_GDGELEMENT *elm = part->element;
        if ( !elm->is_buffer ) continue;

        st_GDGNET *in_net = part->netpin[0];
        st_GDGNET *out_net = part->netpin[1];
        if ( in_net->is_constant ) continue;

/*
        char *partname = gdgpart_create_name ( part );
        printf ( "%s (%d) -> [%s] -> %s (%d)\n", in_net->name, in_net->count_pin, partname, out_net->name, out_net->count_pin );
        g_free ( partname );
*/

        gdgnet_remove_part_pin ( in_net, part->element, "IN", part->loc_x, part->loc_y, 0 );
        gdgnet_remove_part_pin ( out_net, part->element, "OUT", part->loc_x, part->loc_y, 0 );

        st_GDGNET *newnet = NULL;
        st_GDGNET *oldnet = NULL;

        if ( in_net->name[0] != 'F' ) {
            // zachovavame vstupni net
            newnet = in_net;
            oldnet = out_net;
        } else {
            // zachovavame vystupni net
            newnet = out_net;
            oldnet = in_net;
        };
        //printf ( "destroy net: %s\n", oldnet->name );
        gdgnet_merge ( newnet, oldnet );

        int j = remove_part_count++;
        remove_part = g_renew ( st_GDGPART*, remove_part, remove_part_count );
        remove_part[j] = part;

        gdgparts_replace_net ( partall, oldnet, newnet );
        gdgnets_remove_net ( netall, oldnet );
    };

    if ( remove_part_count ) {
        int i;
        for ( i = 0; i < remove_part_count; i++ ) {
            gdgparts_remove_part ( elmall, partall, remove_part[i] );
        };
    };
    g_free ( remove_part );
}


void optimalizer_run ( st_GDGELEMENTSALL *elmall, st_GDGPARTSALL *partall, st_GDGNETSALL *netall ) {
    optimalizer_remove_buffers ( elmall, partall, netall );
}
